% Prob12_8
% Coded by NFR on 10.27.16
% This code uses the equations from prob 12.8 and shows how to use
% fminsearch for NLE
%
% First step, plot to find solution
F1 = @(x) -x^2+x+0.5;
F2 = @(x) x^2/(1+5*x);
% Plot
fplot(F1,[-2 2]);
hold on
fplot(F2,[-2 2]);
% Now that we solved with plot, let's use the fminsearch 
%Initial guess
x0 = 1.2;
y0 = 1.2;
% Combine into single vector
Xo = [x0 y0];
AnsVec = fminsearch(@NLEfind,Xo);
x = AnsVec(1)
y = AnsVec(2)
F1_check = F1(x)
F2_check = F2(x)
% Added on 3.31.19 
% You can also solve this using the newtmult function (included as local
% function below)
% This requires the JACOBEAN matrix to be defined. We evaluatue this using
% the partial derivatives of each equation (NOTE, rearrange each equation
% such that it is equal to zero).
%
% Equation 1, partial derivative with respect to x
%   -2x+1
% Equation 1, partial derivative with respect to y
%   -1
% Equation 2, partial derivative with respect to x
%   -2x+5y
% Equation 2, partial derivative with respect to y
%   1+5x
% Now arrange these in matrix form, first row is first equation, second row
% is the second equation (column 1 = partial derivatives wrt x, column 2 =
% partial derivative wrt y)
%
% J = [-2x+1 -1; -2x+5y 1+5x]
%
% To use newtmult function, this needs to be setup as a subfunction
[x,f,ea,iter]=newtmult(@Jfunc,[x0 y0]');
disp('Answer using Jacobian w/ Newton Raphson:')
x

function [J,f]=Jfunc(x)
% x(1) = x
% x(2) = y
%
f(1,1) = -x(1)^2+x(1)+.5-x(2);
f(2,1) = x(2)+5*x(1)*x(2)-x(1)^2;
J = [-2*x(1)+1 -1; -2*x(1)+5*x(2) 1+5*x(1)];
end

function OUT = NLEfind(X)
x = X(1);
y = X(2);
z1 = -x^2+x+0.5-y;
z2 = x^2-y-5*x*y;
OUT = (z1^2+z2^2)^(1/2);
end

function [x,f,ea,iter]=newtmult(func,x0,es,maxit,varargin)
% newtmult: Newton-Raphson root zeroes nonlinear systems
% [x,f,ea,iter]=newtmult(func,x0,es,maxit,p1,p2,...):
% uses the Newton-Raphson method to find the roots of
% a system of nonlinear equations
% input:
% func = name of function that returns f and J
% x0 = initial guess
% es = desired percent relative error (default = 0.0001%)
% maxit = maximum allowable iterations (default = 50)
% p1,p2,... = additional parameters used by function
% output:
% x = vector of roots
% f = vector of functions evaluated at roots
% ea = approximate percent relative error (%)
% iter = number of iterations
if nargin<2,error('at least 2 input arguments required'),end
if nargin<3|isempty(es),es=0.0001;end
if nargin<4|isempty(maxit),maxit=50;end
iter = 0;
x=x0;
while (1)
[J,f]=func(x,varargin{:});
dx=J\f;
x=x-dx;
iter = iter + 1;
ea=100*max(abs(dx./x));
if iter>=maxit|ea<=es, break, end
end
end
